# -*- coding: utf-8 -*-
"""
Created on Wed Mar 19 16:06:35 2014

@author: Nassim
PyOpenGl texture webcam : idem que test1.py mais l'image stockée dans le numpy array vient de la
webcam.
"""

from OpenGL.GL import *
from OpenGL.GLU import *
import pygame, pygame.image
from pygame.locals import *
from OpenGL.GLUT import *
import pickle
import math
from numpy import *
from VideoCapture import Device


width,height = 1280,720

def setup():
    pygame.init()
    pygame.display.set_mode((width,height),OPENGL | DOUBLEBUF)
    pygame.display.set_caption('Test 2')    

def set_modelview_from_camera(x,y,Rt):
    """ Set the model view matrix from camera pose. """
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()
    # rotate teapot 90 deg around x-axis so that z-axis is up
    Rx = array([[1,0,0],[0,0,-1],[0,1,0]])
    # set rotation to best approximation
    R = Rt[:,:3]
    U,S,V = linalg.svd(R)
    R = dot(U,V)
    R[0,:] = -R[0,:] # change sign of x-axis
    # set translation
    t = Rt[:,3]
    t[0] = x
    t[1] = y
    # setup 4*4 model view matrix
    M = eye(4)
    M[:3,:3] = dot(R,Rx)
    M[:3,3] = t
    # transpose and flatten to get column order
    M = M.T
    m = M.flatten()
    # replace model view with the new matrix
    glLoadMatrixf(m)


def set_projection_from_camera(K):
    """ Set view from a camera calibration matrix. """
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()   
    # set perspective
    #gluPerspective(fovy,aspect,near,far)
    # ou
    glOrtho(-1.0,1.0,-1.0,1.0,-1.0,1.0);
    glViewport(0,0,width,height)


def draw_background(imname):
    """ Draw background image using a quad. """
    # load background image (should be .bmp) to OpenGL texture
    image_surface = pygame.image.frombuffer(imname.tostring(), imname.size, imname.mode)  
    bg_image = image_surface.convert()
    bg_data = pygame.image.tostring(bg_image,"RGBX",1)  
    
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
    # bind the texture
    glEnable(GL_TEXTURE_2D)
    glBindTexture(GL_TEXTURE_2D,glGenTextures(1))
    glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,width,height,0,GL_RGBA,GL_UNSIGNED_BYTE,bg_data)
    glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST)
    glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST)
    # create quad to fill the whole window
    glBegin(GL_QUADS)
    glTexCoord2f(0.0,0.0); glVertex3f(-1.0,-1.0,-1.0)
    glTexCoord2f(1.0,0.0); glVertex3f( 1.0,-1.0,-1.0)
    glTexCoord2f(1.0,1.0); glVertex3f( 1.0, 1.0,-1.0)
    glTexCoord2f(0.0,1.0); glVertex3f(-1.0, 1.0,-1.0)
    glEnd()
    # clear the texture
    glDeleteTextures(1)

   
def draw_cube(size,x,y,angle,Rt): 
    xmin,xmax,ymin,ymax = -1.0,1.0,-1.0,1.0   
    currentPositionX = x * (xmax-xmin) / width + xmin
    y = height - y
    currentPositionY = y * (ymax-ymin) / height + ymin       
    
    glutInit()
    set_modelview_from_camera(currentPositionX,currentPositionY,Rt)
    glPushMatrix()
    glRotate(angle,0,0,1)
    glRotate(angle,0,1,0)
    glColor3f (1.0, 1.0, 1.0);
    # dessin du cube
    glutWireCube(size)
    glPopMatrix()

def reshape(w, h):   
   glMatrixMode(GL_PROJECTION);  
   glLoadIdentity()
   glViewport(0,0,w,h);
    
#main method
def main():  
    #load webcam
    cam = Device()    
    # résolution de la caméra
    cam.setResolution(1280,720) 
    angle = 1
    #load camera
    with open('ar_cameraHD.pkl','r') as f:
        K = pickle.load(f)
        Rt = pickle.load(f)
    setup()
    
    while True:
        # on récupère une image de la webcam
        image = cam.getImage()   
        
        draw_background(image)
        set_projection_from_camera(K)  
        
        # position x et y de la souris
        x,y = pygame.mouse.get_pos()
        
        # affichage du cube
        draw_cube(0.1,x,y,angle,Rt)
        
        # modification de l'angle
        angle = (angle + 1)%90 
        
        reshape(width,height)

        event = pygame.event.poll()
        if event.type in (QUIT,KEYDOWN):
            break         
        
        pygame.display.flip()
        


main()
