# -*- coding: utf-8 -*-
"""
Created on Wed Mar 26 14:06:37 2014

@author: Nassim
"""
from OpenGL.GL import *
from OpenGL.GLU import *
import pygame, pygame.image
from pygame.locals import *
from OpenGL.GLUT import *
import pickle
import math
from numpy import *
import numpy as np

from PIL import Image
import cv2

from PCV.geometry import homography, camera
from PCV.localdescriptors import sift


# taille des images
width,height = 600,600

currentPositionX = 0
currentPositionY = 0

#choix des deux images
image1 = "Images/image1.png"
image2 = "Images/image7.png"


def setup():
    pygame.init()
    pygame.display.set_mode((width,height),OPENGL | DOUBLEBUF)   
    pygame.display.set_caption('Test 5') 


def my_calibration(sz):   
    row,col = sz
    fx = 2555*col/2592
    fy = 2586*row/1936
    K = diag([fx,fy,1])
    K[0,2] = 0.5*col
    K[1,2] = 0.5*row
    return K

def set_modelview_from_camera(Rt,x,y,rot):
    """ Set the model view matrix from camera pose. """
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()   
  
    # set rotation to best approximation
    R = Rt[:,:3]
    U,S,V = linalg.svd(R)
    R = dot(U,V)
    R[0,:] = -R[0,:] # change sign of x-axis
   
   
    # set translation   
    t = Rt[:,3]    
    t[0] = x
    t[1] = y
      
    
    # setup 4*4 model view matrix
    M = eye(4) 
    M[:3,:3] = rot
    M[:3,3]= t
    
    #print M
    
    # transpose and flatten to get column order
    M = M.T
    m = M.flatten()
    
    # replace model view with the new matrix
    glLoadMatrixf(m)


def set_projection_from_camera():
    """ Set view from a camera calibration matrix. """
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity() 
    
    glOrtho(-1.0,1.0,-1.0,1.0,-1.0,1.0);
    glViewport(0,0,width,height) 
    
    
def cube_points(c,wid):
    """ Creates a list of points for plotting
        a cube with plot. (the first 5 points are
        the bottom square, some sides repeated). """
    p = []
    # bottom
    p.append([c[0]-wid,c[1]-wid,c[2]-wid])
    p.append([c[0]-wid,c[1]+wid,c[2]-wid])
    p.append([c[0]+wid,c[1]+wid,c[2]-wid])
    p.append([c[0]+wid,c[1]-wid,c[2]-wid])
    p.append([c[0]-wid,c[1]-wid,c[2]-wid]) #same as first to close plot
    
    # top
    p.append([c[0]-wid,c[1]-wid,c[2]+wid])
    p.append([c[0]-wid,c[1]+wid,c[2]+wid])
    p.append([c[0]+wid,c[1]+wid,c[2]+wid])
    p.append([c[0]+wid,c[1]-wid,c[2]+wid])
    p.append([c[0]-wid,c[1]-wid,c[2]+wid]) #same as first to close plot
    
    # vertical sides
    p.append([c[0]-wid,c[1]-wid,c[2]+wid])
    p.append([c[0]-wid,c[1]+wid,c[2]+wid])
    p.append([c[0]-wid,c[1]+wid,c[2]-wid])
    p.append([c[0]+wid,c[1]+wid,c[2]-wid])
    p.append([c[0]+wid,c[1]+wid,c[2]+wid])
    p.append([c[0]+wid,c[1]-wid,c[2]+wid])
    p.append([c[0]+wid,c[1]-wid,c[2]-wid])
    
    return array(p).T
    
def draw_background(im):
    """ Draw background image using a quad. """
    image_surface = pygame.image.frombuffer(im.tostring(), im.size, im.mode)  
    bg_image = image_surface.convert()
    bg_data = pygame.image.tostring(bg_image,"RGBA",1)  
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
    # bind the texture
    glEnable(GL_TEXTURE_2D)
    glBindTexture(GL_TEXTURE_2D,glGenTextures(1))
    glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,width,height,0,GL_RGBA,GL_UNSIGNED_BYTE,bg_data)
    glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST)
    glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST)
    # create quad to fill the whole window
    glBegin(GL_QUADS)
    glTexCoord2f(0.0,0.0); glVertex3f(-1.0,-1.0,-1.0)
    glTexCoord2f(1.0,0.0); glVertex3f( 1.0,-1.0,-1.0)
    glTexCoord2f(1.0,1.0); glVertex3f( 1.0, 1.0,-1.0)
    glTexCoord2f(0.0,1.0); glVertex3f(-1.0, 1.0,-1.0)
    glEnd()
    # clear the texture
    glDeleteTextures(1)
    
def draw_teapot(size,x,y,rot,Rt):    
    """ Draw a red teapot at the origin. """
    glEnable(GL_LIGHTING)
    glEnable(GL_LIGHT0)
    glEnable(GL_DEPTH_TEST)
    glClear(GL_DEPTH_BUFFER_BIT)
    # draw red teapot

    """
    glMaterialfv(GL_FRONT,GL_AMBIENT,[0,0,0,0])
    glMaterialfv(GL_FRONT,GL_DIFFUSE,[0.5,0.0,0.0,0.0])
    glMaterialfv(GL_FRONT,GL_SPECULAR,[0.7,0.6,0.6,0.0])
    glMaterialf(GL_FRONT,GL_SHININESS,0.25*128.0)   
    """
    glutInit()   
    
    xmin,xmax,ymin,ymax = -1.0,1.0,-1.0,1.0   
    
    
    currentPositionX = x * (xmax-xmin) / width + xmin
    y = height - y
    currentPositionY = y * (ymax-ymin) / height + ymin     

    
    set_modelview_from_camera(Rt,currentPositionX,currentPositionY,rot)
        
    glPushMatrix()    
    glutSolidTeapot(size)
    glPopMatrix()
    
   
def algorithme_sift():   
    #calcul
    mysift = cv2.SIFT(400)
    img0 = cv2.imread(image1,cv2.COLOR_BGR2GRAY)
    l0, d0 = mysift.detectAndCompute(img0,None)
    l0=np.asarray([ [i.pt[0],i.pt[1]] for i in l0]) #convert to array([[x0,y0],[x1,y1],...])
    img1 = cv2.imread(image2,cv2.COLOR_BGR2GRAY)
    l1, d1 = mysift.detectAndCompute(img1,None)
    l1=np.asarray([ [i.pt[0],i.pt[1]] for i in l1]) #convert to array([[x0,y0],[x1,y1],...])
    
    
    # match features and estimate homography
    matches = sift.match_twosided(d0,d1)
    ndx = matches.nonzero()[0]
    fp = homography.make_homog(l0[ndx,:2].T) 
    ndx2 = [int(matches[i]) for i in ndx]
    tp = homography.make_homog(l1[ndx2,:2].T)
    
    model = homography.RansacModel()
    H, inliers = homography.H_from_ransac(fp,tp,model) 
    K = my_calibration((height,width)) 
    
    # 3D points at plane z=0 with sides of length 0.2
    box = cube_points([0,0,0.1],0.1)
    
    # project bottom square in first image
    cam1 = camera.Camera( hstack((K,dot(K,array([[0],[0],[-1]])) )) )
    # first points are the bottom square
    box_cam1 = cam1.project(homography.make_homog(box[:,:5])) 
    
    
    # use H to transfer points to the second image
    box_trans = homography.normalize(dot(H,box_cam1))    
    
    # compute second camera matrix from cam1 and H
    cam2 = camera.Camera(dot(H,cam1.P))
    A = dot(linalg.inv(K),cam2.P[:,:3])
    A = array([A[:,0],A[:,1],cross(A[:,0],A[:,1])]).T 
    cam2.P[:,:3] = dot(K,A)
    
    # project with the second camera
    box_cam2 = cam2.project(homography.make_homog(box))  
    #print H
    
    return box_cam1,box_cam2,H
    
def reshape(w, h):
   glViewport(0,0,w,h);
   glMatrixMode(GL_PROJECTION);  
   glLoadIdentity()
    
def main():
    
    with open('ar_cameraHD.pkl','r') as f:    
        K = pickle.load(f)
        Rt = pickle.load(f)
     
    # initialisation de pygame
    setup()
    
    im1 = Image.open(image1)  
    im2 = Image.open(image2)
    
    # application de l'algorithme sift et récupération de la matrice H
    box_cam1,box_cam2,H = algorithme_sift()   
    
    print H
    
    # on dessine la première image de fond
    draw_background(im1) 
    
    # projection from camera
    set_projection_from_camera() 
       
    x = (box_cam1[0,0]+box_cam1[0,2])/2
    y = (box_cam1[1,0]+box_cam1[1,2])/2        
    
    # on dessine la teapot au milieu de l'écran 
    rot_init = array([[1,0,0],[0,1,0],[0,0,1]])
    draw_teapot(0.1,x,y,rot_init,Rt)
    
    pygame.display.flip()
    
    # on attend 5 secondes
    pygame.time.wait(5000)   
    
    reshape(width,height)
    
    # on dessine la deuxième image de fond
    draw_background(im2)   
    
     # projection from camera
    set_projection_from_camera()   
   
     #calculer le centre
    x = (box_cam2[0,0]+box_cam2[0,2])/2
    y = (box_cam2[1,0]+box_cam2[1,2])/2   
    
    # on créé la matrice de rotation grâce à H
    rot = array([[H[0,0],H[0,1],0],[H[1,0],H[1,1],0],[H[2,0],H[2,1],H[2,2]]])
    
    draw_teapot(0.1,x,y,rot,Rt) 
    
    pygame.display.flip()  
   
    while True:
        event = pygame.event.poll()
        if event.type in (QUIT,KEYDOWN):
            break
        
        
    
main()