# -*- coding: utf-8 -*-
"""
Created on Tue Apr 15 09:59:59 2014

@author: ben
"""

from numpy import * 
import numpy as np
import sift
import cv2
import cv
from pylab import *
import skimage
from PCV.geometry import homography, camera
from PCV.localdescriptors import sift

def numpyrgb2opencvrgb(numpyrgb):
    cvbgr=cv.fromarray(numpyrgb.copy())
    cvrgb = cv.CreateImage(cv.GetSize(cvbgr),8,3)
    cv.CvtColor(cvbgr, cvrgb, cv.CV_RGB2BGR)
    return cvrgb

def cube_points(c,wid):
    """ Creates a list of points for plotting
        a cube with plot. (the first 5 points are
        the bottom square, some sides repeated). """
    p = []
    # bottom
    p.append([c[0]-wid,c[1]-wid,c[2]-wid])
    p.append([c[0]-wid,c[1]+wid,c[2]-wid])
    p.append([c[0]+wid,c[1]+wid,c[2]-wid])
    p.append([c[0]+wid,c[1]-wid,c[2]-wid])
    p.append([c[0]-wid,c[1]-wid,c[2]-wid]) #same as first to close plot
    
    # top
    p.append([c[0]-wid,c[1]-wid,c[2]+wid])
    p.append([c[0]-wid,c[1]+wid,c[2]+wid])
    p.append([c[0]+wid,c[1]+wid,c[2]+wid])
    p.append([c[0]+wid,c[1]-wid,c[2]+wid])
    p.append([c[0]-wid,c[1]-wid,c[2]+wid]) #same as first to close plot
    
    # vertical sides
    p.append([c[0]-wid,c[1]-wid,c[2]+wid])
    p.append([c[0]-wid,c[1]+wid,c[2]+wid])
    p.append([c[0]-wid,c[1]+wid,c[2]-wid])
    p.append([c[0]+wid,c[1]+wid,c[2]-wid])
    p.append([c[0]+wid,c[1]+wid,c[2]+wid])
    p.append([c[0]+wid,c[1]-wid,c[2]+wid])
    p.append([c[0]+wid,c[1]-wid,c[2]-wid])
    
    return array(p).T

def my_calibration(sz):
    """
    Calibration function for the camera (iPhone4) used in this example.
    """
    row,col = sz
    fx = 2555*col/2592
    fy = 2586*row/1936
    K = diag([fx,fy,1])
    K[0,2] = 0.5*col
    K[1,2] = 0.5*row
    return K
    

def operation(img0,img1):
    
    mysift = cv2.SIFT(400)
    #img0 = cv2.imread('../data/test.png',cv2.COLOR_BGR2GRAY)
    l0, d0 = mysift.detectAndCompute(img0,None)
    l0=np.asarray([ [i.pt[0],i.pt[1]] for i in l0]) #convert to array([[x0,y0],[x1,y1],...])
    #img1 = cv2.imread('../data/test2.png',cv2.COLOR_BGR2GRAY)
    l1, d1 = mysift.detectAndCompute(img1,None)
    l1=np.asarray([ [i.pt[0],i.pt[1]] for i in l1]) #convert to array([[x0,y0],[x1,y1],...])
    
    
    #match features and estimate homography
    matches = sift.match_twosided(d0,d1)
    ndx = matches.nonzero()[0]
    fp = homography.make_homog(l0[ndx,:2].T) 
    ndx2 = [int(matches[i]) for i in ndx]
    tp = homography.make_homog(l1[ndx2,:2].T)
    
    model = homography.RansacModel()
    H, inliers = homography.H_from_ransac(fp,tp,model) 
    # camera calibration
    height=img0.shape[0]
    width=img0.shape[1]
    
    K = my_calibration((height,width))
    
    # 3D points at plane z=0 with sides of length 0.2
    box = cube_points([0,0.1,0],0.1)
    
    # project bottom square in first image
    cam1 = camera.Camera( hstack((K,dot(K,array([[0],[0],[-1]])) )) )
    # first points are the bottom square
    box_cam1 = cam1.project(homography.make_homog(box[:,:5])) 
    
    
    # use H to transfer points to the second image
    box_trans = homography.normalize(dot(H,box_cam1))
    
    # compute second camera matrix from cam1 and H
    cam2 = camera.Camera(dot(H,cam1.P))
    A = dot(linalg.inv(K),cam2.P[:,:3])
    A = array([A[:,0],A[:,1],cross(A[:,0],A[:,1])]).T 
    cam2.P[:,:3] = dot(K,A)
    
    # project with the second camera
    box_cam2 = cam2.project(homography.make_homog(box))   
    
    # plotting
    im0 = array(img0)
    im1 = array(img1)

    cv_rgb=numpyrgb2opencvrgb(img1) 

    cv.Rectangle(cv_rgb, (int(box_trans[0,0]),int(box_trans[1,0])),(int(box_trans[0,2]),int(box_trans[1,2])), cv.RGB(255,0,0), thickness=5)
    
    # Convert cv image to array    
    im3 = np.asarray( cv_rgb[:,:] )
    
    #imsave("opencv.png",im3)
    
    '''figure()
    imshow(im0)
    plot(box_cam1[0,:],box_cam1[1,:],linewidth=3)
    title('2D projection of bottom square')
    axis('off')
    savefig("imageTest1.png",bbox_inches='tight')
    
    
    
    figure()
    imshow(im1)
    plot(box_trans[0,:],box_trans[1,:],linewidth=3)
    title('2D projection transfered with H')
    axis('off')
    savefig("imageTest2.png",bbox_inches='tight')
    
    
    figure()
    imshow(im1)
    plot(box_cam2[0,:],box_cam2[1,:],linewidth=3)
    title('3D points projected in second image')
    axis('off')
    savefig("imageTest3.png",bbox_inches='tight')
    
    #show()'''
    return cv_rgb

def calculateHomography(image, template):
    l0,l1,d0,d1 = matchPointFromImage(image, template)

    # match features and estimate homography
    matches = sift.match_twosided(d0,d1)
    ndx = matches.nonzero()[0]

    # print ndx
    # initialize an array with x in the first column and y in the 
    # second column of the first image

    src_points = []
    for i in ndx:
        src_points.append(l0[i].pt)

    src_points = array(src_points)
    ndx2 = [int(matches[i]) for i in ndx]

    # initialize an array with x in the first column and y in the 
    # second column of the second image
    target_points = []
    for j in ndx2:
        target_points.append(l1[j].pt)
    
    target_points = array(target_points)
    # Mise en forme selon page 66: version optimise
    f_src_points=src_points.flatten()
    A=zeros((len(f_src_points),4))
    #Colonne 0
    A[::2,0]=f_src_points[::2] #les x: tous les pairs
    A[1::2,0]=f_src_points[1::2] #les y : tous les impairs
    #Colonne 1
    A[::2,1]=-f_src_points[1::2] #les -y : tous les impairs
    A[1::2,1]=f_src_points[::2] #les x : tous les pairs
    #Colonne 2 & 3: des 1 sur lignes pairs/impaires
    A[::2,2]=1.0
    A[1::2,3]=1.0
    B=target_points.flatten()

    #print A, "\n", B

    #Resultat
    a,b,tx,ty = linalg.lstsq(A,B)[0]

    ######################################
    #EXPLOITATION: mise en forme M, T et H + affichage
    ######################################
    M=array([[a,-b],
             [b,a]])
    print "\nMatrice rotation-homothetie:\n", M
    T=array([tx,ty])
    print "\nMatrice translation:\n", T
    H=array([[a,-b,tx],
             [b,a,ty],
             [0,0,1]])
    print "\nHomography:\n", H
    
    return H
    
def matchPointFromImage(img1, template):
    #compute features with opencv
    image = cv2.cvtColor(img1,cv2.COLOR_BGR2GRAY)
    siftalg = cv2.SIFT(contrastThreshold=0.09)
    kp,des = siftalg.detectAndCompute(image,None)
    img = cv2.drawKeypoints(image,kp)
    cv2.imwrite('sift_keypoints.jpg',img)

    temp = cv2.cvtColor(template,cv2.COLOR_BGR2GRAY)
    kp2,des2 = siftalg.detectAndCompute(temp,None)
    temp = cv2.drawKeypoints(temp,kp2)
    cv2.imwrite('sift_keypoints_perspective.jpg',temp)

    # Normalisation des vecteurs
    for i in range(des.shape[0]):
        des[i,:] /= linalg.norm(des[i,:])
 
    for i in range(des2.shape[0]):
        des2[i,:] /= linalg.norm(des2[i,:])
    #print des,des2
    
    # match features and estimate homography
    #matches = s.match(des,des2)


    # Matching Keypoints using FLANN
    flann_params = dict(algorithm=1, trees=4)
    flann = cv2.flann_Index(des, flann_params)
    idx, dist = flann.knnSearch(des2, 1, params={})
    del flann

    # Sorting keypoint matches according to the distance:
    dist = dist[:,0]/2500.0
    dist = dist.reshape(-1,).tolist()
    idx = idx.reshape(-1).tolist()
    indices = range(len(dist))
    indices.sort(key=lambda i: dist[i])
    dist = [dist[i] for i in indices]
    idx = [idx[i] for i in indices]

    # filter keypoints by setting a cut-off limit of distance:
    '''kp_final = []
    for i, dis in itertools.izip(idx, dist):
        if dis < 0.7:
            kp_final.append(kp[i])
        else:
            break
    
    tkp_final = []
    for i, dis in itertools.izip(idx, dist):
        if dis < 0.7:
            tkp_final.append(kp2[i])
        else:
            break'''
        
    kp_final = kp
    tkp_final = kp2

    # Draw Keypoints: 
    h1, w1 = img.shape[:2]
    h2, w2 = temp.shape[:2]
    nWidth = w1+w2
    nHeight = max(h1, h2)
    hdif = (h1-h2)/2
    newimg = zeros((nHeight, nWidth, 3), uint8)
    newimg[hdif:hdif+h2, :w2] = temp
    newimg[:h1, w2:w1+w2] = img

    # draw lines joining matched points.
    kp2 = tkp_final
    kp = kp_final


    for i in range(min(len(kp2), len(kp))):
        pt_a = (int(kp2[i].pt[0]), int(kp2[i].pt[1]+hdif))
        pt_b = (int(kp[i].pt[0]+w2), int(kp[i].pt[1]))
        cv2.line(newimg, pt_a, pt_b, (255, 0, 0))

    imsave("iii.jpg",newimg)
    #cv2.imshow("image", newimg)
    cv2.waitKey(0)
    
    
    return kp_final, tkp_final, des, des2